import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import $ from 'jquery';
import RU from '../../utils/ramda-utils';
import {isDisabled} from '../../utils/components-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import {sortStrings} from '../../utils/sorting-utils';
import {computeNewStateOnNewKeyBlur, isKeyUnique} from '../input/input-service';
import {XlReactDipSetOfStringsComponent} from './xl-react-dip-set-of-strings-component';
import * as service from './xl-react-dip-set-of-strings-service';
import {getUpdatedCellState} from './set-of-strings-utils';

const delayTime = 200;

const resetAllErrors = {
    duplicateKey: undefined,
    emptyKey: undefined,
    hasNewKeyError: undefined
};

export class XlReactDipSetOfStrings extends Component {
    static propTypes = {
        model: PropTypes.array,
        pristine: PropTypes.bool,
        showNewKeyValueRow: PropTypes.bool,
        valid: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        displayMode: 'edit',
        metadata: {},
        validators: []
    };

    static getDerivedStateFromProps(props, state) {
        if (state.pristine) {
            const modelKeys = service.getModelKeys(props.model, true);
            return {modelKeys};
        }
        return null;
    }

    constructor(props) {
        super(props);
        this.state = {
            newKey: '',
            searchWord: '',
            showNewKeyValueRow: false
        };
        RU.invoke('register', [this.api], props);
    }

    handleCleaningSearchWord = () => {
        this.setState({searchWord: ''});
    };

    handleKeyBlur = (changedKey, oldKey) => {
        const {model} = this.props;
        if (!R.contains(oldKey, model)) {
            return;
        }

        const trimmedChangedKey = R.trim(changedKey);
        const keyChanged = !R.equals(trimmedChangedKey, oldKey);

        const handleChangedKey = () => {
            if (!trimmedChangedKey) {
                this.setState({emptyKey: true});
            } else if (!isKeyUnique(trimmedChangedKey, oldKey)(model)) {
                const newState = {duplicateKey: trimmedChangedKey};
                this.setState(newState);
            } else {
                const updatedModel = service.getModelWithUpdatedKey(model, oldKey, trimmedChangedKey);

                const focusToInputField = () => {
                    if (this.state.hasNewKeyError) {
                        this.delayedInputKeyFocus();
                    } else {
                        this.focusKeyInput(trimmedChangedKey);
                    }
                };
                this.applyNewState(updatedModel, focusToInputField);
            }
        };

        if (!keyChanged) {
            if (this.state.hasNewKeyError) {
                this.delayedInputKeyFocus();
            } else {
                this.setState(resetAllErrors);
            }
        } else {
            handleChangedKey();
        }
    };

    handleRemoveKey = (key) => {
        const model = R.difference(this.props.model, [key]);
        this.applyNewState(model, this.focusToInputKeyWithError);
    };

    handleSearchChanged = (searchWord) => {
        this.setState({searchWord});
    };

    handleSortChange = ({sortBy, sortDirection}) => {
        this.setState({
            cellState: getUpdatedCellState(this.props.model, this.state.cellState, {sortDirection}),
            sorting: {sortBy, sortDirection}
        });
    };

    onCloseButton = () => {
        this.setState({
            ...resetAllErrors,
            newKey: '',
            showNewKeyValueRow: false
        });
    };

    onAddRow = () => setTimeout(() => {
        if (!this.state.hasNewKeyError) {
            this.setState({
                cellState: {
                    column: 'key',
                    row: 0,
                    value: ''
                },
                searchWord: '',
                showNewKeyValueRow: true
            }, () => {
                this.tableRef.scrollToRow(0);
                this.delayedInputKeyFocus();
            });
        }
    }, delayTime);

    onNewKeyBlurDelayedHandler = () => {
        const newState = computeNewStateOnNewKeyBlur(this.state, this.props.model);
        const applyModelIfPossible = () => {
            const {newKey, newValueFocused} = this.state;
            const trimmedNewKey = R.trim(newKey);

            if (!newValueFocused && trimmedNewKey) {
                this.addModelKey();
            }
        };
        this.setState({...newState}, applyModelIfPossible);
    };

    onNewKeyBlur = () => {
        setTimeout(this.onNewKeyBlurDelayedHandler, delayTime);
    };

    onNewKeyChange = (event) => {
        this.setState({newKey: event.target.value});
    };

    onNewKeyFocus = () => {
        this.setState(resetAllErrors);
    };

    delayedInputKeyFocus = () => setTimeout(() => {
        this.inputKeyField.focus();
    }, delayTime);

    addModelKey = () => {
        const {newKey} = this.state;
        const {model} = this.props;
        const keyIsUnique = isKeyUnique(newKey)(model);
        if (keyIsUnique) {
            this.setState({
                newKey: '',
                showNewKeyValueRow: false
            });
            const focusToField = () => {
                this.focusKeyInput(newKey);
            };
            this.applyNewState([...model, newKey], focusToField);
        } else {
            this.setState({
                duplicateKey: newKey,
                emptyKey: undefined,
                hasNewKeyError: true
            });
        }
    };

    focusKeyInput = (key) => {
        const tableInputs = R.values($(this.tableWrapperRef)
            .find('.key-div input'));
        const mapKeyInputField = R.find(R.propSatisfies(R.equals(key), 'defaultValue'), tableInputs);
        if (mapKeyInputField) {
            mapKeyInputField.focus();
        }
    };

    focusToInputKeyWithError = () => {
        if (this.state.hasNewKeyError) {
            this.delayedInputKeyFocus();
        }
    };

    api = {
        focus: () => {
            this.setState({showNewKeyValueRow: true}, () => {
                this.delayedInputKeyFocus();
            });
        }
    };

    applyNewState = (model, callback = R.F) => {
        this.props.onModelChange(model);
        this.setState({emptyKey: undefined}, callback);
    };

    assignRef = (input) => {
        this.inputKeyField = input;
    };

    updateCellState = (cellState) => {
        this.setState({cellState});
    };

    assignTableRef = (table) => {
        this.tableRef = table;
    };

    assignTableWrapperRef = (tableWrapper) => {
        this.tableWrapperRef = tableWrapper;
    };

    render() {
        const {
            displayMode,
            message,
            metadata,
            model,
            onValidityChange,
            pristine,
            valid
        } = this.props;
        const {
            cellState,
            duplicateKey,
            emptyKey,
            newKey,
            showNewKeyValueRow,
            searchWord,
            sorting
        } = this.state;

        const {
            autoHide, canAddNewItem, description, label, password, required
        } = metadata;

        const disabled = isDisabled(this.props.disabled, displayMode);

        return <XlReactDipSetOfStringsComponent {...({
            assignRef: this.assignRef,
            assignTableRef: this.assignTableRef,
            assignTableWrapperRef: this.assignTableWrapperRef,
            autoHide,
            canAddNewItem,
            cellState,
            description,
            disabled,
            duplicateKey,
            emptyKey,
            encrypted: password,
            handleAddRow: this.onAddRow,
            handleCloseButton: this.onCloseButton,
            handleNewKeyBlur: this.onNewKeyBlur,
            handleNewKeyChange: this.onNewKeyChange,
            handleNewKeyFocus: this.onNewKeyFocus,
            handleSortChange: this.handleSortChange,
            label,
            message,
            model: sortStrings(model, sorting ? sorting.sortDirection : 'ASC'),
            newKey,
            onCleaningSearchWord: this.handleCleaningSearchWord,
            onKeyBlur: this.handleKeyBlur,
            onRemoveKey: this.handleRemoveKey,
            onSearchChanged: this.handleSearchChanged,
            onValidityChange,
            pristine,
            required,
            searchWord,
            showNewKeyValueRow,
            sorting,
            updateCellState: this.updateCellState,
            valid
        })}
        />;
    }
}
