import PropTypes from 'prop-types';
import R from 'ramda';
import RS from 'ramdasauce';
import React, {Component} from 'react';
import uuid from 'uuid-v4';
import {
    computeInitialState,
    computeNextPropsState,
    computeNextState,
    getValidators
} from '../../services/component-service';
import {getModelValue, isDisabled, resetPristinity} from '../../utils/components-utils';
import RU from '../../utils/ramda-utils';
import {xlMessageTypeProp} from '../../xl-props/xl-message-type-prop';
import {XlReactWidgetContainerProps} from '../xl-react-widget-props';
import {autocompleteSuggestionBehaviorPropsType} from './components/autocomplete-suggestion-behavior-type';
import {defaultSuggestionBehavior} from './components/default-suggestion-behavior';
import {XlReactWidgetAutocompleteComponent} from './xl-react-widget-autocomplete-component';

export const DEFAULT_OPTION_HEIGHT = 30;
export const DEFAULT_MENU_HEIGHT = 240;

export class XlReactWidgetAutocomplete extends Component {
    static propTypes = {
        addons: PropTypes.oneOfType([
            PropTypes.node,
            PropTypes.arrayOf(PropTypes.node)
        ]),
        className: PropTypes.string,
        clearInputOnBlur: PropTypes.bool,
        controlClassName: PropTypes.string,
        disableSorting: PropTypes.bool,
        handlers: PropTypes.object,
        inputProps: PropTypes.object,
        menuHeight: PropTypes.number,
        message: xlMessageTypeProp,
        messageHandler: PropTypes.func,
        metadata: PropTypes.object,
        model: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.object
        ]),
        objLabel: PropTypes.string,
        onSuggestionNotFound: PropTypes.func,
        onSuggestionSelected: PropTypes.func,
        optionHeight: PropTypes.number,
        renderInput: PropTypes.func,
        renderSuggestion: PropTypes.func,
        required: PropTypes.bool,
        showSearchIcon: PropTypes.bool,
        subActions: PropTypes.arrayOf(PropTypes.shape({
            action: PropTypes.func.isRequired,
            isDisabled: PropTypes.func.isRequired,
            label: PropTypes.string.isRequired
        })),
        suggestionBehavior: autocompleteSuggestionBehaviorPropsType,
        ...XlReactWidgetContainerProps
    };

    static defaultProps = {
        disableSorting: false,
        menuHeight: DEFAULT_MENU_HEIGHT,
        metadata: {},
        objLabel: 'id',
        optionHeight: DEFAULT_OPTION_HEIGHT,
        showSearchIcon: true,
        validators: []
    };

    constructor(props) {
        super(props);
        this.state = {
            id: uuid(),
            loadingSuggestions: false,
            suggestions: this.props.suggestions || [],
            typing: false
        };
    }

    UNSAFE_componentWillMount() {
        const model = getModelValue(this.props, 'model', '');
        computeInitialState(this, this.props, getValidators(this.props), model);
    }

    UNSAFE_componentWillReceiveProps(nextProps) {
        computeNextPropsState(this, nextProps, this.state, getValidators(nextProps));
    }

    componentWillUnmount() {
        RU.invoke('unregister', [], this.props);
    }

    handleChange = (event, {newValue}) => {
        computeNextState(this, this.props, getValidators(this.props), newValue);
    };

    handleErase = () => {
        this.componentApi.erase();
        const {model} = this.props;
        const emptyValue = R.is(Object, model) ? {} : '';
        computeNextState(this, this.props, getValidators(this.props), emptyValue);
    };

    handleOnBlur = () => {
        this.setState({loadingSuggestions: false});
    };

    handleSuggestionsClearRequested = () => {
        this.setState({suggestions: []});
    };

    onGetSuggestions = (val, reason) => {
        const {disableSorting, handlers, metadata, objLabel} = this.props;
        if (handlers) {
            const addCandidates = handlers.addCandidates;
            if (R.isNil(addCandidates)) {
                throw new Error('Handler addCandidates has to be defined');
            }

            const toSearch = () => this.state.model !== val || !this.state.pristine ? val : '';
            const findSuggestions = (suggestions) => {
                const sortByObjLabel = R.sortBy(R.compose(R.toLower, R.prop(objLabel)));
                const orderedSuggestion = disableSorting ? suggestions : sortByObjLabel(suggestions);
                this.setState({suggestions: [...orderedSuggestion]});
            };

            this.setState({loadingSuggestions: true}, () => {
                addCandidates({metadata, objLabel, reason, value: toSearch()})
                    .then((suggestions) => {
                        RU.invoke('onCheckSuggestionExists', [this.hasSuggestion(suggestions, val)], this.props);
                        findSuggestions(suggestions);
                    })
                    .finally(() => {
                        this.setState({loadingSuggestions: false});
                    });
            });
        }
    };

    onSuggestionSelected = (event, {suggestion}, callbacks) => {
        RU.invoke('onSuggestionSelected', [suggestion, this.state.suggestions, callbacks, event], this.props);
    };

    hasSuggestion = (suggestions, inputValue) => {
        const {objLabel} = this.props;

        const suggestionLabels = R.map(
            R.when(
                R.has(objLabel),
                R.prop(objLabel)
            ))(suggestions);

        const value = inputValue || R.ifElse(
            R.has(objLabel),
            R.prop(objLabel),
            R.identity
        )(this.state.model);

        return RS.isNilOrEmpty(value) || R.includes(value, suggestionLabels);
    };

    clearSuggestions = () => {
        this.setState({suggestions: []});
    };

    registerComponentApi = (api) => {
        this.componentApi = api;
    };

    api = {
        blur: () => this.autocomplete.onBlur(),
        erase: this.handleErase,
        focus: () => this.autocomplete.input.focus(),
        getSuggestions: () => this.state.suggestions,
        resetPristinity: (config) => resetPristinity(this, this.props, config),
        revealSuggestions: () => {
            this.onGetSuggestions('');
            this.autocomplete.revealSuggestions();
            this.autocomplete.input.focus();
        }
    };

    assignRef = (autocomplete) => {
        this.autocomplete = autocomplete;
    };

    render() {
        const {
            className, controlClassName, clearInputOnBlur, disabled, displayMode, messageHandler,
            inputProps, menuHeight, metadata, objLabel,
            onPristinityChange, onValidityChange, optionHeight,
            addons, renderInput, renderSuggestion, showSearchIcon, suggestionBehavior, subActions, type
        } = this.props;
        const {description, label, required, requiredDirection, placeholder} = metadata;
        const {
            loadingSuggestions,
            message,
            id: componentId,
            model,
            suggestions,
            valid
        } = this.state;

        return <XlReactWidgetAutocompleteComponent {...({
            addons,
            assignRef: this.assignRef,
            className,
            clearInputOnBlur,
            clearSuggestions: this.clearSuggestions,
            componentId,
            controlClassName,
            description,
            disabled: isDisabled(disabled, displayMode),
            handleGetSuggestions: this.onGetSuggestions,
            handleSuggestionSelected: this.onSuggestionSelected,
            inputProps,
            label,
            loadingSuggestions,
            menuHeight,
            message,
            messageHandler,
            model,
            objLabel,
            onBlur: this.handleOnBlur,
            onChange: this.handleChange,
            onErase: this.handleErase,
            onPristinityChange,
            onSuggestionsClearRequested: this.handleSuggestionsClearRequested,
            onValidityChange,
            optionHeight,
            placeholder,
            register: this.registerComponentApi,
            renderInput,
            renderSuggestion,
            required,
            requiredDirection,
            showSearchIcon,
            subActions,
            suggestionBehavior: suggestionBehavior || defaultSuggestionBehavior,
            suggestions,
            type,
            valid
        })}
        />;
    }
}
