import React, {useState} from 'react';
import PropTypes from 'prop-types';
import ReactResizeDetector from 'react-resize-detector';
import RU from '../../utils/ramda-utils';

const propagateEditorDidMount = (editor, props) => RU.invoke(['editorDidMount'], [editor], props);

export const withEditorAutoFocus = (WrappedComponent) => {
    const component = React.forwardRef((props, ref) => {
        const {focusOnMount, ...componentProps} = props;

        const onEditorMount = (editor) => {
            if (focusOnMount) {
                editor.focus();
            }
            propagateEditorDidMount(editor, props);
        };

        return <WrappedComponent
            {...componentProps}
            editorDidMount={onEditorMount}
            ref={ref}
        />;
    });
    component.propTypes = {
        editorDidMount: PropTypes.func,
        focusOnMount: PropTypes.bool
    };
    return component;
};

export const withEditorResize = (WrappedComponent) => {
    const component = React.forwardRef((props, ref) => {
        const {resizeFrequency, ...componentProps} = props;
        const [editorRef, setEditorRef] = useState();

        const onEditorMount = (editor) => {
            setEditorRef(editor);
            propagateEditorDidMount(editor, props);
        };

        const resizeWindowHandler = () => {
            if (editorRef) {
                editorRef.layout();
            }
        };

        return <>
            <WrappedComponent
                {...componentProps}
                editorDidMount={onEditorMount}
                ref={ref}
            />
            <ReactResizeDetector
                handleHeight={true}
                handleWidth={true}
                onResize={resizeWindowHandler}
                refreshMode="debounce"
                refreshRate={resizeFrequency}
            />
        </>;
    });

    component.propTypes = {
        editorDidMount: PropTypes.func,
        resizeFrequency: PropTypes.number
    };

    component.defaultProps = {resizeFrequency: 100};
    return component;
};

export const withoutContextMenu = (WrappedComponent) =>
    React.forwardRef((props, ref) => {
        const onEditorMount = (editor) => {
            editor.updateOptions({contextmenu: false});
            propagateEditorDidMount(editor, props);
        };

        return <WrappedComponent
            {...props}
            editorDidMount={onEditorMount}
            ref={ref}
        />;
    });
