import R from 'ramda';
import RU from '~/app/utils/ramda-utils';

describe('Ramda utils', () => {
    describe('invoke', () => {
        it('should invoke the function with specified parameters', () => {
            const object = {bar: {foo: jest.fn()}};
            RU.invoke(['bar', 'foo'], [1, 2, 3])(object);
            expect(object.bar.foo.mock.calls).toEqual([[1, 2, 3]]);
        });

        it('should not fail when pointing to wrong path', () => {
            const object = {bar: {foo: jest.fn()}};
            RU.invoke(['baz', 'foo'], [1, 2, 3])(object);
            expect(object.bar.foo.mock.calls).toEqual([]);
        });

        it('should invoke function with just a single string path', () => {
            const object = {foo: jest.fn()};
            RU.invoke('foo', [1, 2, 3])(object);
            expect(object.foo.mock.calls).toEqual([[1, 2, 3]]);
        });

        it('should invoke function without parameters', () => {
            const object = {foo: jest.fn()};
            RU.invoke('foo', [])(object);
            expect(object.foo.mock.calls).toEqual([[]]);
        });

        it('should invoke function by path without parameters', () => {
            const object = {bar: {foo: jest.fn()}};
            RU.invoke(['bar', 'foo'], [])(object);
            expect(object.bar.foo.mock.calls).toEqual([[]]);
        });

        it('should not fail when pointing to wrong path without parameters', () => {
            const object = {bar: {foo: jest.fn()}};
            RU.invoke(['baz', 'foo'], [])(object);
            expect(object.bar.foo.mock.calls).toEqual([]);
        });
    });

    describe('indexedArray', () => {
        it('should index array by creating an array of arrays putting index as a second item in sub-array', () => {
            expect(RU.indexedArray(['a', 'b', 'c'])).toEqual([['a', 0], ['b', 1], ['c', 2]]);
        });
    });

    describe('incrementProp', () => {
        it('should increment specified property', () => {
            expect(RU.incrementProp('propA', {propA: 2, propB: 2})).toEqual({propA: 3, propB: 2});
        });

        it('should default properties to 0 if they was not defined on object yet and then increment', () => {
            expect(RU.incrementProp('propC', {propA: 2, propB: 2})).toEqual({propA: 2, propB: 2, propC: 1});
        });
    });

    describe('countHits', () => {
        it('should count amount of hits using functions defined in specification object', () => {
            const input = [
                {propA: 1, propB: true, propC: 'A'},
                {propA: -20, propB: false, propC: 'B'},
                {propA: 33, propB: false, propC: 'A'}
            ];
            const result = RU.countHits({
                cEqLowerA: R.propSatisfies(R.pipe(R.toLower, R.equals('a')), 'propC'),
                positiveA: R.propSatisfies(R.gt(R.__, 0), 'propA'),
                trueB: R.propEq('propB', true)
            })(input);
            expect(result).toEqual({cEqLowerA: 2, positiveA: 2, trueB: 1});
        });

        it('should return empty values if condition never happened', () => {
            const input = [
                {propA: 1, propB: true, propC: 'A'},
                {propA: -20, propB: false, propC: 'B'},
                {propA: 33, propB: false, propC: 'A'}
            ];
            const result = RU.countHits({
                trueB: R.propEq('propB', true),
                trueC: R.propEq('propC', true)
            })(input);
            expect(result).toEqual({trueB: 1, trueC: 0});
        });
    });

    describe('isTrue', () => {
        it('should return true if property value is true in the object', () => {
            const object = {property: true};
            expect(RU.isTrue(['property'], object)).toEqual(true);
        });

        it('should return false if property value is false in the object', () => {
            const object = {property: false};
            expect(RU.isTrue(['property'], object)).toEqual(false);
        });

        it('should return false if that property is not present in the object', () => {
            const object = {propertyz: true};
            expect(RU.isTrue(['property'], object)).toEqual(false);
        });

        it('should return false if that object is undefined', () => {
            const object = undefined;
            expect(RU.isTrue(['property'], object)).toEqual(false);
        });
    });

    describe('isFalse', () => {
        it('should return false if property value is true in the object', () => {
            const object = {property: true};
            expect(RU.isFalse(['property'], object)).toEqual(false);
        });

        it('should return true if property value is false in the object', () => {
            const object = {property: false};
            expect(RU.isFalse(['property'], object)).toEqual(true);
        });

        it('should return true if that property is not present in the object', () => {
            const object = {propertyz: true};
            expect(RU.isFalse(['property'], object)).toEqual(true);
        });

        it('should return true if that object is undefined', () => {
            const object = undefined;
            expect(RU.isFalse(['property'], object)).toEqual(true);
        });
    });

    describe('areDefinedAndDifferent', () => {
        it('should return false if at least one of value is not defined', () => {
            expect(RU.areDefinedAndDifferent(undefined, undefined)).toEqual(false);
            expect(RU.areDefinedAndDifferent(1, undefined)).toEqual(false);
            expect(RU.areDefinedAndDifferent(undefined, 2)).toEqual(false);
            expect(RU.areDefinedAndDifferent(null, null)).toEqual(false);
            expect(RU.areDefinedAndDifferent(null, undefined)).toEqual(false);
            expect(RU.areDefinedAndDifferent(undefined, null)).toEqual(false);
        });
        it('should return false if both values are defined and equal', () => {
            expect(RU.areDefinedAndDifferent(2, 2)).toEqual(false);
            expect(RU.areDefinedAndDifferent(false, false)).toEqual(false);
        });
        it('should return true if both values are defined and not equal', () => {
            expect(RU.areDefinedAndDifferent(1, 2)).toEqual(true);
            expect(RU.areDefinedAndDifferent(false, true)).toEqual(true);
        });
    });

    describe('findDuplicateElement', () => {
        it('should find duplicate elements', () => {
            expect(RU.findDuplicateElement([{value: 1}, {value: 1}])).toEqual('1');
            expect(RU.findDuplicateElement([{value: 1}, {value: 2}])).toBeUndefined();
        });
    });
});
