import React from 'react';
import R from 'ramda';
import {mount} from 'enzyme';
import {Promise} from 'es6-promise';
import {waitFor} from 'async-enzyme-helpers';
import {XlReactDipListOfCi} from '~/app/xl-react-dip/xl-react-dip-list-of-ci/xl-react-dip-list-of-ci';

const getListOfCiHandlers = () => ({
    addCandidates: () => Promise.resolve(),
    onClick: () => Promise.resolve(),
    onNewItem: () => Promise.resolve()
});

const selector = 'SortableList';

describe('xl-dip-list-of-ci', () => {
    const model = ['Item 1', 'Item 2', 'Item 3', 'Item 4'];
    const visibleItems = R.addIndex(R.map)((item, idx) => ({model: [item, idx], valid: true}), model);

    describe('XlReactDipListOfCi-default', () => {
        const defaultProps = {
            handlers: getListOfCiHandlers(),
            metadata: {
                description: 'test desc',
                label: 'test label',
                name: 'test name',
                required: false
            },
            model: []
        };

        it('should have all elements present', () => {
            const onModelChange = jest.fn();
            const onValidityChange = jest.fn();
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                context={{searchTerm: '', visibleItems}}
                model={model}
                onModelChange={onModelChange}
                onValidityChange={onValidityChange}
            />);
            expect(wrapper.find('XlReactDipCollectionOfCi')).toExist();
            expect(wrapper.find('XlReactDipCollectionOfCi .xl-react-autocomplete-container')).toExist();
            expect(wrapper.find('XlReactWidgetLabel')).toExist();
            expect(wrapper.find('XlReactWidgetLabel label')).toHaveText(defaultProps.metadata.label);
            expect(wrapper.find('.xl-react-components-description')).toExist();
            expect(wrapper.find('.xl-react-components-description')).toHaveText(defaultProps.metadata.description);
            expect(wrapper.find('XlReactWidgetAutocomplete')).toExist();
            expect(wrapper.find('XlReactWidgetFilter')).toExist();
            expect(onModelChange).toBeCalledWith(['Item 1', 'Item 2', 'Item 3', 'Item 4'], false);
            expect(wrapper.find('SortableList').prop('canSort')).toEqual(true);
            expect(wrapper.find('SortableList ul')).toHaveClassName('ui-sortable');
            const items = wrapper.find('SortableList .xl-list-row');
            expect(items.length).toEqual(4);

            const input = wrapper.find('XlReactDipCollectionOfCi .xl-react-autocomplete-container input[type="text"]');
            expect(input).toExist();
            input.simulate('change', {target: {value: 'A'}});
            expect(wrapper.find('XlReactWidgetAutocomplete .close-icon')).toExist();
        });

        it('should have delete buttons for all list items', () => {
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                context={{searchTerm: '', visibleItems}}
                model={model}
            />);
            const items = wrapper.find('SortableList .xl-list-row .close-icon');
            expect(items.length).toEqual(4);
        });

        it('should trigger remove element', async () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                context={{searchTerm: '', visibleItems}}
                model={['Item 1', 'Item 2', 'Item 3', 'Item 4']}
                onModelChange={onModelChange}
            />);
            expect(onModelChange).toBeCalledWith(['Item 1', 'Item 2', 'Item 3', 'Item 4'], false);
            const deleteButtons = () => wrapper.find(`${selector} .xl-list-row .close-icon`);
            await waitFor(wrapper, selector);

            deleteButtons().first().simulate('click');
            await waitFor(wrapper, selector);
            expect(onModelChange).toBeCalledWith(['Item 2', 'Item 3', 'Item 4']);
        });

        it('should clear input field with close button', () => {
            const wrapper = mount(<XlReactDipListOfCi {...defaultProps}/>);
            const input = wrapper.find('XlReactDipCollectionOfCi .xl-react-autocomplete-container input[type="text"]');
            expect(input).toExist();
            input.simulate('change', {target: {value: 'AA'}});
            expect(wrapper.find('XlReactWidgetAutocomplete .close-icon')).toExist();
            input.simulate('change', {target: {value: ''}});
            expect(wrapper.find('XlReactWidgetAutocomplete .close-icon')).not.toExist();
        });

        it('should verify that filter is visible in view mode', () => {
            const wrapper = mount(<XlReactDipListOfCi {...defaultProps} displayMode="edit"/>);
            expect(wrapper.find('XlReactWidgetFilter input[type="text"]')).toExist();
        });

        it('should filter items', () => {
            const mergeContext = jest.fn();
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                context={{searchTerm: '', validatedItems: visibleItems, visibleItems}}
                mergeContext={mergeContext}
                model={model}
            />);
            const filterInput = wrapper.find('XlReactWidgetFilter input[type="text"]');
            const items = () => wrapper.find('SortableList .xl-list-row');
            expect(items().length).toEqual(4);

            filterInput.simulate('change', {target: {value: '1'}});
            expect(mergeContext).toBeCalledWith({searchTerm: '1', visibleItems: [{model: ['Item 1', 0], valid: true}]});

            filterInput.simulate('change', {target: {value: 'item'}});
            expect(mergeContext).toBeCalledWith({searchTerm: 'item', visibleItems});

            filterInput.simulate('change', {target: {value: 'zzz'}});
            expect(mergeContext).toBeCalledWith({searchTerm: 'zzz', visibleItems: []});
        });

        it('should display all item when filter is emptied', () => {
            const mergeContext = jest.fn();
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                context={{searchTerm: 'item21', validatedItems: visibleItems, visibleItems: []}}
                mergeContext={mergeContext}
                model={model}
            />);
            const filterInput = wrapper.find('XlReactWidgetFilter input[type="text"]');
            const items = () => wrapper.find('SortableList .xl-list-row');
            expect(items().length).toEqual(0);

            filterInput.simulate('change', {target: {value: ''}});
            expect(mergeContext).toBeCalledWith({searchTerm: '', visibleItems});
        });

        it('should/nt be disabled', () => {
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                disabled={false}
            />);
            expect(wrapper.find('Autosuggest input[disabled="true"]')).not.toExist();
            expect(wrapper.find('SortableList ul')).toHaveClassName('ui-sortable');
            wrapper.setProps({disabled: true});
            expect(wrapper.find('.xl-react-autocomplete-container input[disabled]')).toExist();
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');
        });
    });

    describe('XlReactDipListOfCi-displayMode', () => {
        const defaultProps = {
            handlers: getListOfCiHandlers(),
            metadata: {
                default: ['item1', 'item3', 'item3'],
                description: 'test desc',
                label: 'test label',
                name: 'test name',
                required: false
            }
        };

        it('should switch display mode', () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                context={{searchTerm: '', visibleItems}}
                model={model}
                onModelChange={onModelChange}
            />);
            const input = wrapper.find('.xl-react-autocomplete-container input[type="text"]');
            const deleteButtons = () => wrapper.find('SortableList .xl-list-row .close-icon');
            wrapper.setProps({displayMode: ''});
            expect(input).toExist();
            expect(deleteButtons().length).toBe(4);

            wrapper.setProps({displayMode: 'edit'});
            expect(wrapper.find('SortableList ul')).toHaveClassName('ui-sortable');
            expect(input).toExist();
            expect(deleteButtons().length).toBe(4);

            wrapper.setProps({displayMode: 'visual'});
            expect(wrapper.find('.xl-react-autocomplete-container input[type="text"][disabled]')).toExist();
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');
            expect(wrapper.find('.xl-btn-add')).not.toExist();

            wrapper.setProps({
                disabled: false,
                displayMode: 'edit'
            });
            wrapper.setProps({disabled: true});
            expect(wrapper.find('.xl-react-autocomplete-container input[type="text"][disabled]')).toExist();
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');
            expect(deleteButtons().length).toBe(0);
        });

        it('should not be able to edit in Visual mode', () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                context={{searchTerm: '', visibleItems}}
                displayMode="visual"
                model={model}
                onModelChange={onModelChange}
            />);
            const input = wrapper.find('.xl-react-autocomplete-container input[type="text"][disabled]');
            const deleteButtons = () => wrapper.find('SortableList .xl-list-row .close-icon');
            const rows = wrapper.find('SortableList .xl-list-row');

            expect(input).toExist();
            expect(rows.length).toEqual(4);
            expect(deleteButtons()).not.toExist();
        });

        it('should be able to filter list in Visual mode', () => {
            const onModelChange = jest.fn();
            const mergeContext = jest.fn();
            const wrapper = mount(<XlReactDipListOfCi
                {...defaultProps}
                context={{searchTerm: '', validatedItems: visibleItems, visibleItems}}
                displayMode="visual"
                mergeContext={mergeContext}
                model={model}
                onModelChange={onModelChange}
            />);
            const input = wrapper.find('.xl-react-autocomplete-container input[type="text"][disabled]');
            const deleteButtons = () => wrapper.find('SortableList .xl-list-row .close-icon');
            const items = () => wrapper.find('SortableList .xl-list-row');
            const filterInput = wrapper.find('XlReactWidgetFilter input[type="text"]');

            expect(input).toExist();
            expect(items().length).toEqual(4);
            expect(deleteButtons()).not.toExist();
            expect(wrapper.find('SortableList').prop('canSort')).toEqual(false);
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');

            filterInput.simulate('change', {target: {value: '1'}});
            expect(mergeContext).toBeCalledWith({searchTerm: '1', visibleItems: [{model: ['Item 1', 0], valid: true}]});

            filterInput.simulate('change', {target: {value: 'item'}});
            expect(mergeContext).toBeCalledWith({searchTerm: 'item', visibleItems});

            filterInput.simulate('change', {target: {value: 'zzz'}});
            expect(mergeContext).toBeCalledWith({searchTerm: 'zzz', visibleItems: []});

            filterInput.simulate('change', {target: {value: ''}});
            expect(mergeContext).toBeCalledWith({searchTerm: '', visibleItems});
        });
    });
});
