import React from 'react';
import {mount} from 'enzyme';

import Helper from '../../validations-spec-helper';
import {XlReactDipList} from '~/app/xl-react-dip/xl-react-dip-list/xl-react-dip-list';
import {ERROR} from '~/app/utils/ci-utils';

describe('xl-dip-list', () => {
    describe('XlReactDipList-default', () => {
        const defaultProps = {
            metadata: {
                default: ['item1', 'item3', 'item3'],
                description: 'test desc',
                label: 'test label',
                name: 'test name',
                required: false
            }
        };

        it('should have all elements to be present', () => {
            const onModelChange = jest.fn();
            const onValidityChange = jest.fn();
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
                onValidityChange={onValidityChange}
            />);
            expect(wrapper.find('SortableList ul')).toHaveClassName('ui-sortable');
            const items = wrapper.find('ul > li.xl-list-row');
            expect(items.length).toEqual(defaultProps.metadata.default.length);
        });

        it('should add element and add button click', () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
            />);
            const input = () => wrapper.find('XlReactWidgetString input[type="text"]');
            input().simulate('change', {target: {value: 'test-value'}});
            expect(input()).toHaveValue('test-value');
            wrapper.find('.xl-btn-add').simulate('click');
            setTimeout(() => {
                expect(input()).toHaveValue('');
                expect(onModelChange).toBeCalledWith([...defaultProps.metadata.default, 'test-value'], false);
            }, 500);
        });

        it('should add element when no suggestion available', () => {
            const handlers = {'test name': {addCandidates: () => Promise.resolve([])}};
            const onModelChange = jest.fn();
            const mergeContext = jest.fn();
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                handlers={handlers}
                mergeContext={mergeContext}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
            />);
            const autoSuggest = wrapper.find('XlReactWidgetAutocomplete');
            expect(autoSuggest).toExist();
            const input = wrapper.find('XlReactDipList .xl-react-autocomplete-container input[type="text"]');
            input.simulate('change', {target: {value: 'test-value'}});
            expect(mergeContext).toBeCalledWith({typed: 'test-value'});
            wrapper.setProps({context: {typed: 'test-value'}});
            expect(wrapper.find('XlReactWidgetAutocomplete .close-icon')).toExist();
            wrapper.find('.xl-btn-add').simulate('click');
            expect(mergeContext).toBeCalledWith({typed: ''});
            expect(input).toHaveValue('');
            wrapper.setProps({context: {typed: ''}});
            expect(onModelChange).toBeCalledWith([...defaultProps.metadata.default, 'test-value']);
        });

        it('should not add empty element', () => {
            const onModelChange = jest.fn();
            const mergeContext = jest.fn();
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                mergeContext={mergeContext}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
            />);
            const input = wrapper.find('XlReactWidgetString input[type="text"]');
            input.simulate('change', {target: {value: ''}});
            expect(input).toHaveValue('');
            wrapper.find('.xl-btn-add').simulate('click');
            expect(input).toHaveValue('');
            expect(onModelChange).toBeCalledTimes(0);
        });

        it('should add 2 times the same element', () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
            />);
            const input = wrapper.find('XlReactWidgetString input[type="text"]');

            input.simulate('change', {target: {value: 'same-same'}});
            wrapper.find('.xl-btn-add').simulate('click');

            input.simulate('change', {target: {value: 'same-same'}});
            wrapper.find('.xl-btn-add').simulate('click');

            setTimeout(() => {
                expect(onModelChange).toBeCalledWith(
                    [...defaultProps.metadata.default, 'same-same', 'same-same'], false);
            }, 500);
        });

        it('should trigger remove element', () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={['item1', 'item2', 'item3']}
                onModelChange={onModelChange}
            />);
            const deleteButtons = () => wrapper.find('SortableList .xl-list-row .close-icon');
            deleteButtons().first().simulate('click');
            expect(onModelChange).toBeCalledWith(['item2', 'item3']);
        });

        it('should honour custom validators', () => {
            const validator = (model) => ({
                condition: model.length > 4,
                message: {show: false, text: 'At most 4 items allowed', type: ERROR}
            });
            const {changeInput, errMsg, wrapper} = new Helper('.xl-react-dip-list').buildFrom(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                validators={[validator]}
            />);

            changeInput('item4');
            wrapper.find('.xl-btn-add').simulate('click');
            expect(errMsg()).not.toExist();

            changeInput('item5');
            wrapper.find('.xl-btn-add').simulate('click');
            setTimeout(() => {
                expect(errMsg()).toHaveText('At most 4 items allowed');
            }, 500);
        });

        it('dip list component WITH label', () => {
            const onModelChange = jest.fn();
            const onValidityChange = jest.fn();

            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
                onValidityChange={onValidityChange}
            />);

            expect(wrapper.find('XlReactWidgetLabel label')).toExist();
            expect(wrapper.find('XlReactWidgetLabel label')).toHaveText(defaultProps.metadata.label);
        });

        it('dip list component WITHOUT label', () => {
            const defaultProps1 = {
                metadata: {
                    default: ['item1', 'item3', 'item3'],
                    description: 'test desc',
                    name: 'test name',
                    required: false
                }
            };
            const onModelChange = jest.fn();
            const onValidityChange = jest.fn();

            const wrapper = mount(<XlReactDipList
                {...defaultProps1}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
                onValidityChange={onValidityChange}
            />);
            expect(wrapper.find('XlReactWidgetLabel label')).not.toExist();
        });
    });

    describe('XlReactDipList-required', () => {
        const defaultProps = {
            metadata: {
                default: ['item1', 'item3', 'item3'],
                description: 'test desc',
                label: 'test label',
                name: 'test name',
                required: true
            }
        };

        it('should have all elements to be present', () => {
            const onModelChange = jest.fn();
            const onValidityChange = jest.fn();
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
                onValidityChange={onValidityChange}
            />);
            const items = wrapper.find('ul > li.xl-list-row');
            expect(items.length).toEqual(defaultProps.metadata.default.length);
        });

        it('should add an element', () => {
            const onModelChange = jest.fn();
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
            />);
            const input = wrapper.find('XlReactWidgetString input[type="text"]');

            input.simulate('change', {target: {value: 'react'}});
            wrapper.find('.xl-btn-add').simulate('click');

            setTimeout(() => {
                expect(onModelChange).toBeCalledWith([...defaultProps.metadata.default, 'react'], false);
            }, 500);
        });
    });

    describe('XlReactDipList-displayMode', () => {
        const defaultProps = {
            metadata: {
                default: ['item1', 'item3', 'item3'],
                description: 'test desc',
                label: 'test label',
                name: 'test name',
                required: false
            }
        };
        const onModelChange = jest.fn();

        it('should switch display mode', () => {
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
            />);

            const input = wrapper.find('XlReactWidgetString input[type="text"]');
            const addButton = wrapper.find('.xl-btn-add');
            wrapper.setProps({displayMode: ''});
            expect(input).toExist();
            expect(addButton).toExist();
            expect(wrapper.find('SortableList ul')).toHaveClassName('ui-sortable');

            wrapper.setProps({displayMode: 'edit'});
            expect(input).toExist();
            expect(addButton).toExist();
            expect(wrapper.find('SortableList ul')).toHaveClassName('ui-sortable');

            wrapper.setProps({displayMode: 'visual'});
            expect(wrapper.find('XlReactWidgetString input[type="text"]')).not.toExist();
            expect(wrapper.find('.xl-btn-add')).not.toExist();
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');

            wrapper.setProps({
                disabled: false,
                displayMode: 'edit'
            });
            wrapper.setProps({disabled: true});
            expect(wrapper.find('XlReactWidgetString input[type="text"][readOnly=true]')).toExist();
            expect(wrapper.find('SortableList ul')).not.toHaveClassName('ui-sortable');
        });

        it('Visual mode should not allow any user editing', () => {
            const wrapper = mount(<XlReactDipList
                {...defaultProps}
                model={defaultProps.metadata.default}
                onModelChange={onModelChange}
            />);
            const sortableList = () => wrapper.find('SortableList');
            wrapper.setProps({displayMode: 'visual'});
            const input = wrapper.find('XlReactWidgetString input[type="text"]');
            expect(sortableList().find('ul')).not.toHaveClassName('ui-sortable');
            expect(input).not.toExist();

            const addButton = wrapper.find('.xl-btn-add');
            expect(addButton).not.toExist();

            const ListItems = wrapper.find('SortableList .xl-list-row');
            expect(ListItems.length).toEqual(3);

            const removeButtons = wrapper.find('SortableList .xl-list-row .close-icon');
            expect(removeButtons.length).toEqual(0);

            expect(sortableList().prop('canSort')).toEqual(false);
            expect(sortableList().find('ul')).not.toHaveClassName('ui-sortable');
        });
    });
});
