import R from 'ramda';
import metadata from './application-metadata.json';
import {groupedProperties} from './grouped-properties.json';
import {groupedProperties as updatedGroupedProperties} from './updated-grouped-properties.json';
import {groupedProperties as openedGroupedProperties} from './expanded-grouped-properties.json';
import {groupedProperties as collapsedGroupedProperties} from './collapsed-grouped-properties.json';
import formActions
    from '~/app/xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-actions';
import {xlReactDipReduxFormReducer}
    from '~/app/xl-react-form/xl-react-dip-redux-form/xl-react-dip-redux-form-reducer';

describe('xl-react-dip-redux-form-reducer', () => {
    describe('xlReactDipReduxFormReducer', () => {
        it('should initialize redux store with initial form data', () => {
            const result = xlReactDipReduxFormReducer({})({}, formActions.initialize({
                collapsed: 'auto',
                formId: 'form-1',
                metadata,
                pristine: true
            }));
            expect(R.assocPath(['form-1', 'context'], undefined, result)).toEqual({
                'form-1': {
                    collapsed: 'auto',
                    context: undefined,
                    formId: 'form-1',
                    groupedProperties: R.map((prop) => ({
                        ...prop,
                        categoryOptions: undefined
                    }), collapsedGroupedProperties),
                    handlers: {},
                    ignoredProperties: undefined,
                    message: {show: false},
                    messages: {
                        lastVersion: undefined,
                        pipeline: undefined,
                        verifyChecklistPermissionsOnCreate: undefined
                    },
                    metadata,
                    model: {
                        lastVersion: '',
                        pipeline: '',
                        verifyChecklistPermissionsOnCreate: false
                    },
                    objLabel: 'id',
                    pristinity: {
                        lastVersion: true,
                        pipeline: true,
                        verifyChecklistPermissionsOnCreate: true
                    },
                    validators: [],
                    validity: {
                        lastVersion: true,
                        pipeline: true,
                        verifyChecklistPermissionsOnCreate: true
                    }
                }
            });
        });
    });
    describe('mergeContext', () => {
        it('should merge new context with already existing one', () => {
            const updatedMetadata = {
                ...metadata,
                properties: R.map((property) => ({...property, kind: 'BOOLEAN'}), metadata.properties)
            };

            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    metadata
                }
            }, formActions.mergeContext('form-1',
                {metadata: updatedMetadata}
            ))).toEqual({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: updatedGroupedProperties,
                    metadata: updatedMetadata
                }
            });
        });
    });
    describe('removeValidationMessages', () => {
        it('should remove validation message by property name', () => {
            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: 'auto',
                    formId: 'form-1',
                    groupedProperties: collapsedGroupedProperties,
                    handlers: {},
                    metadata,
                    model: {
                        'lastVersion': '',
                        'pipeline': '',
                        'validation-messages': [{
                            ci: 'Applications/AppForHistory/1.0/asdf',
                            level: 'WARNING',
                            message: 'Value is required',
                            property: 'lastVersion'
                        }, {
                            ci: 'Applications/AppForHistory/1.0/asdf',
                            level: 'WARNING',
                            message: 'Value is required',
                            property: 'pipeline'
                        }],
                        'verifyChecklistPermissionsOnCreate': false
                    },
                    objLabel: 'id',
                    pristinity: {
                        lastVersion: true,
                        pipeline: true,
                        verifyChecklistPermissionsOnCreate: true
                    }
                }
            }, formActions.removeValidationMessages('form-1', ['lastVersion', 'pipeline']))).toEqual({
                'form-1': {
                    collapsed: 'auto',
                    formId: 'form-1',
                    groupedProperties: collapsedGroupedProperties,
                    handlers: {},
                    metadata,
                    model: {
                        lastVersion: '',
                        pipeline: '',
                        verifyChecklistPermissionsOnCreate: false
                    },
                    objLabel: 'id',
                    pristinity: {
                        lastVersion: true,
                        pipeline: true,
                        verifyChecklistPermissionsOnCreate: true
                    }
                }
            });
        });
    });
    describe('openCategoryGroup', () => {
        it('should open all category group in grouped properties', () => {
            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties,
                    metadata
                }
            }, formActions.openCategoryGroup('form-1', 'Common'))).toEqual({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                }
            });
        });
    });
    describe('toggleCategoryGroup', () => {
        it('should toggle category group in grouped properties', () => {
            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                }
            }, formActions.toggleCategoryGroup('form-1', 'Common'))).toEqual({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties,
                    metadata
                }
            });
        });
    });
    describe('unmount', () => {
        it('should unmount all data for specified form', () => {
            expect(xlReactDipReduxFormReducer({})({
                'form-1': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                },
                'form-2': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                }
            }, formActions.unmount('form-1'))).toEqual({
                'form-2': {
                    collapsed: true,
                    formId: 'form-1',
                    groupedProperties: openedGroupedProperties,
                    metadata
                }
            });
        });
    });
    describe('changePristineState', () => {
        it('should set pristine state for specified form', () => {
            const form1 = {
                collapsed: true,
                formId: 'form-1',
                groupedProperties: openedGroupedProperties,
                metadata
            };

            const pristinity1 = {condition: false, path: true};
            const pristinity2 = {condition: true, path: true};

            expect(xlReactDipReduxFormReducer({})(
                {'form-1': form1},
                formActions.changePristineState('form-1', pristinity1))
            ).toEqual({'form-1': {...form1, pristinity: pristinity1}});

            expect(xlReactDipReduxFormReducer({})(
                {'form-1': form1},
                formActions.changePristineState('form-1', pristinity2))
            ).toEqual({'form-1': {...form1, pristinity: pristinity2}});
        });
    });
    describe('changeValidity', () => {
        it('should set validity for specified form', () => {
            const form1 = {
                collapsed: true,
                formId: 'form-1',
                groupedProperties: openedGroupedProperties,
                metadata
            };

            const validity1 = {condition: false, path: true};
            const validity2 = {condition: true, path: true};

            expect(xlReactDipReduxFormReducer({})(
                {'form-1': form1},
                formActions.changeValidity('form-1', validity1))
            ).toEqual({'form-1': {...form1, validity: validity1}});

            expect(xlReactDipReduxFormReducer({})(
                {'form-1': form1},
                formActions.changeValidity('form-1', validity2))
            ).toEqual({'form-1': {...form1, validity: validity2}});
        });
    });
});
