import R from 'ramda';
import {
    hasSearchedTerm,
    toggleChecked,
    filterSuggestionsByValue,
    suggestionToModel,
    keepOrRemoveColor,
    selectRemainingSuggestions,
    transformModelForInputTags,
    appendNewSuggestion,
    removeExistingSuggestion,
    getIndex,
    getItemsToSelect,
    sortItems,
    valueToModel
} from '~/app/xl-react-widget/xl-react-widget-tags/xl-react-widget-tags-service';
import {getBaseName} from '~/app/utils/ci-utils';

const model = R.map(
    (value) => ({label: value, value}),
    ['App1', 'App2']
);
const objectLabel = 'id';
const tagTransform = getBaseName;

describe('xl-react-widget-tags-service', () => {
    describe('hasSearchedTerm', () => {
        it('should determine the suggestion has the searched term irrespective of case', () => {
            expect(hasSearchedTerm(objectLabel, 'app', {id: 'Item 1'})).toBe(false);
            expect(hasSearchedTerm(objectLabel, 'app', {id: 'App1'})).toBe(true);
            expect(hasSearchedTerm(objectLabel, 'app', 'App1')).toBe(true);
            expect(hasSearchedTerm(undefined, 'app', 'App1')).toBe(true);
        });
    });

    describe('toggleChecked', () => {
        it('should set checked to true if suggestion is present in the model and vice versa', () => {
            const suggestion1 = {id: 'App1'};
            const suggestion2 = {id: 'App21'};
            expect(toggleChecked(model, objectLabel, suggestion1)).toEqual({checked: true, id: 'App1'});
            expect(toggleChecked(model, objectLabel, suggestion2)).toEqual({checked: false, id: 'App21'});
        });
    });

    describe('valueToModel', () => {
        it('should create a model for a tag component from a provided value', () => {
            expect(valueToModel('Apple')).toEqual({label: 'Apple', value: 'Apple'});
        });
    });

    describe('filterSuggestionsByValue', () => {
        it('should filter out the model values and send back the rest', () => {
            const suggestions = [{id: 'App1'}, {id: 'App2'}, {id: 'App3'}, {id: 'App4'}];
            expect(filterSuggestionsByValue(R.map(
                (value) => ({label: value, value}),
                ['App1', 'App2']
            ), objectLabel, suggestions)).toEqual([{id: 'App3'}, {id: 'App4'}]);
            expect(filterSuggestionsByValue([], objectLabel, suggestions))
                .toEqual([{id: 'App1'}, {id: 'App2'}, {id: 'App3'}, {id: 'App4'}]);
            expect(filterSuggestionsByValue(R.map(
                (value) => ({label: value, value}),
                ['App1', 'App2', 'App3', 'App4']
            ), objectLabel, suggestions))
                .toEqual([]);
        });
    });

    describe('suggestionToModel', () => {
        it('should convert suggestion object to model object', () => {
            expect(suggestionToModel(tagTransform, objectLabel, {id: 'App1'})).toEqual({label: 'App1', value: 'App1'});
            expect(suggestionToModel(tagTransform, objectLabel, {color: '#000AAA', id: 'App1'}))
                .toEqual({color: '#000AAA', label: 'App1', value: 'App1'});
        });
    });

    describe('keepOrRemoveColor', () => {
        it('should remove color if it is undefined', () => {
            expect(keepOrRemoveColor({color: undefined, label: 'App1', value: 'App1'}))
                .toEqual({label: 'App1', value: 'App1'});
            expect(keepOrRemoveColor({color: '#000AAA', label: 'App1', value: 'App1'}))
                .toEqual({color: '#000AAA', label: 'App1', value: 'App1'});
        });
    });

    describe('selectRemainingSuggestions', () => {
        it('should select remaining suggestions aprat from the model values', () => {
            expect(selectRemainingSuggestions(
                objectLabel,
                R.map(
                    (value) => ({id: value, label: value, value}),
                    ['App1']
                ),
                [{id: 'App1'}, {id: 'App2'}, {id: 'App3'}],
                tagTransform
            )).toEqual([
                {label: 'App2', value: 'App2'},
                {label: 'App3', value: 'App3'}
            ]);
            expect(selectRemainingSuggestions(
                objectLabel,
                [],
                [{id: 'App1'}, {id: 'App2'}, {id: 'App3'}],
                tagTransform
            )).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App1', 'App2', 'App3']
            ));
            expect(selectRemainingSuggestions(
                objectLabel,
                R.map(
                    (value) => ({id: value, label: value, value}),
                    ['App1', 'App2', 'App3']
                ),
                [{id: 'App1'}, {id: 'App2'}, {id: 'App3'}],
                tagTransform
            )).toEqual([]);
        });
    });

    describe('appendNewSuggestion', () => {
        it('should append items to the model', () => {
            expect(appendNewSuggestion({id: 'App3'},
                model,
                objectLabel,
                tagTransform)).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App1', 'App2', 'App3']
            ));
            expect(appendNewSuggestion({id: 'App1'},
                [],
                objectLabel,
                tagTransform)).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App1']
            ));
            expect(appendNewSuggestion({id: 'App1'}, R.map(
                (value) => ({label: value, value}),
                ['App2', 'App3']
            ), objectLabel, tagTransform)).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App2', 'App3', 'App1']
            ));
        });
    });

    describe('removeExistingSuggestion', () => {
        it('should append items to the model', () => {
            expect(removeExistingSuggestion({id: 'App1'}, R.map(
                (value) => ({label: value, value}),
                ['App1', 'App2', 'App3']
            ), objectLabel, tagTransform)).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App2', 'App3']
            ));

            expect(removeExistingSuggestion({id: 'App1'}, R.map(
                (value) => ({label: value, value}),
                ['App1']
            ), objectLabel, tagTransform)).toEqual(R.map(
                (value) => ({label: value, value}),
                []
            ));
        });
    });

    describe('transformModelForInputTags', () => {
        it('should fix the model structure for input tags', () => {
            expect(transformModelForInputTags([
                {label: 'App1', value: 'App1'},
                'App2'
            ])).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App1', 'App2']
            ));
            expect(transformModelForInputTags([
                'App1'
            ])).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App1']
            ));
            expect(transformModelForInputTags(R.map(
                (value) => ({label: value, value}),
                ['App1', 'App2']
            ))).toEqual(R.map(
                (value) => ({label: value, value}),
                ['App1', 'App2']
            ));
        });
    });

    describe('getIndex', () => {
        const suggestions = R.map(
            (value) => ({id: value}),
            ['App1', 'App2', 'App3', 'App4']
        );
        it('should get the index of the searched item', () => {
            expect(getIndex({id: 'App2'}, suggestions, objectLabel)).toEqual(1);
            expect(getIndex({id: 'App1'}, suggestions, objectLabel)).toEqual(0);
            expect(getIndex({id: 'App4'}, suggestions, objectLabel)).toEqual(3);
            expect(getIndex({id: 'App5'}, suggestions, objectLabel)).toEqual(-1);
        });
    });

    describe('getItemsToSelect', () => {
        const suggestions = sortItems(objectLabel, R.map((ind) => ({id: `App${ind}`}), R.range(0, 40)));
        it('should get range of items based on indexes', () => {
            expect(getItemsToSelect(0, 3, suggestions)).toEqual([{id: 'App1'}, {id: 'App10'}, {id: 'App11'}]);
            expect(getItemsToSelect(3, 0, suggestions)).toEqual([{id: 'App10'}, {id: 'App1'}, {id: 'App0'}]);
            expect(getItemsToSelect(3, 3, suggestions)).toEqual([{id: 'App11'}]);
            expect(getItemsToSelect(0, 0, suggestions)).toEqual([{id: 'App0'}]);
            expect(getItemsToSelect(39, 39, suggestions)).toEqual([{id: 'App9'}]);
            expect(getItemsToSelect(36, 39, suggestions)).toEqual([{id: 'App7'}, {id: 'App8'}, {id: 'App9'}]);
            expect(getItemsToSelect(39, 36, suggestions)).toEqual([{id: 'App8'}, {id: 'App7'}, {id: 'App6'}]);
        });
    });
});
